from .base import BaseLayer


__all__ = ('LSTM', 'GRU')


class LSTM(BaseLayer):
    """
    Long Short Term Memory (LSTM) Layer.
    """


class GRU(BaseLayer):
    """
    Gated Recurrent Unit (GRU) Layer.
    """
